package polytech.android.e6201;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.media.AudioManager;
import android.media.ToneGenerator;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.NumberPicker;
import android.widget.TextView;

public class TimerActivity extends AppCompatActivity implements View.OnClickListener, NumberPicker.OnValueChangeListener {
    private NumberPicker minNumberPicker;
    private NumberPicker secNumberPicker;
    private TextView timeLeftTextView;
    private Button startButton;
    private Button stopButton;
    private Button backButton;
    private int countSecond;  // タイマーカウントする秒数
    Intent intent = new Intent();  // タイマーサービスの操作用インテント

    // タイマーの残り時間を受信するための、ブロードキャストレシーバ
    private BroadcastReceiver receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int timeLeft = intent.getIntExtra("count", 0);

            if (timeLeft <= 0) {
                // 音を鳴らす
                ToneGenerator toneGenerator = new ToneGenerator(AudioManager.STREAM_SYSTEM, ToneGenerator.MAX_VOLUME);
                toneGenerator.startTone(ToneGenerator.TONE_SUP_PIP, 2000);  // 2秒

                // ボタンの調整（有効／無効）
                startButton.setEnabled(true);
                stopButton.setEnabled(false);
            }
            showTimeLeft(timeLeft);
        }
    };

    // タイマーの残り時間を表示するメソッド
    private void showTimeLeft(int sec) {
        String s = String.valueOf(sec / 60) + "分" + String.valueOf(sec % 60) + "秒";
        timeLeftTextView.setText(s);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_timer);

        // Buttonの設定
        startButton = (Button) findViewById(R.id.startButton);
        startButton.setOnClickListener(this);
        stopButton = (Button) findViewById(R.id.stopButton);
        stopButton.setOnClickListener(this);
        backButton = (Button) findViewById(R.id.backButton4);
        backButton.setOnClickListener(this);

        // NumberPickerの設定
        minNumberPicker = (NumberPicker) findViewById(R.id.minNumberPicker);
        minNumberPicker.setMaxValue(10);
        minNumberPicker.setOnValueChangedListener(this);
        secNumberPicker = (NumberPicker) findViewById(R.id.secNumberPicker);
        secNumberPicker.setMaxValue(59);
        secNumberPicker.setOnValueChangedListener(this);

        // TextView
        timeLeftTextView = (TextView) findViewById(R.id.timeLeftTextView);
        showTimeLeft(countSecond);

        // ブロードキャストレシーバの設定
        IntentFilter filter = new IntentFilter();
        filter.addAction(TimerService.ACTION_TIME_LEFT);
        filter.addCategory(Intent.CATEGORY_DEFAULT);
        registerReceiver(receiver, filter);
    }

    @Override
    public void onClick(View v) {
        if (v == backButton) {  //「戻る」ボタン
            finish();
        } else {

            intent.setClass(getApplicationContext(), TimerService.class);

            if (v == startButton) {  //「スタート」ボタン
                // 設定時間の表示
                countSecond = minNumberPicker.getValue() * 60 + secNumberPicker.getValue();
                showTimeLeft(countSecond);

                if (countSecond > 0) {  // 設定が0秒より大きければ動作
                    // カウント秒数の設定と、サービスの起動
                    intent.putExtra("countSecond", countSecond);
                    startService(intent);

                    // ボタンの調整（有効／無効）
                    startButton.setEnabled(false);
                    stopButton.setEnabled(true);
                }
            } else if (v == stopButton) {  //「ストップ」ボタン
                // サービスの停止
                stopService(intent);

                // ボタンの調整（有効／無効）
                startButton.setEnabled(true);
                stopButton.setEnabled(false);
            }
        }
    }

    @Override
    public void onValueChange(NumberPicker picker, int oldVal, int newVal) {
        countSecond = minNumberPicker.getValue() * 60 + secNumberPicker.getValue();
        showTimeLeft(countSecond);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();

        // サービスの停止（「戻る」ボタン、画面回転の場合を想定）
        if(intent != null){
            stopService(intent);
        }

        // ブロードキャストレシーバの解除
        unregisterReceiver(receiver);
    }
}
